function G_ExtensionUpdater(name) {
if(!name) {
G_Debug(this, "You need to specify an extension name");
return false;
}
this.RDFSvc_  = Cc["@mozilla.org/rdf/rdf-service;1"]
.getService(Ci.nsIRDFService);
this.name = name;
this.updating = false;
}
G_ExtensionUpdater.prototype.Update = function() {
if (this.updating) {
return false;
}
this.updating = true;
this.currentVersion = "";
this.currentUpdateURL = "";
this.updateLink = "";
this.updateVersion = "";
if (!this.GetCurrent()) {
this.Failure("'%s' not found in extension manager.".subs(this.name));
return false;
}
this.AttemptUpdate();
}
G_ExtensionUpdater.prototype.CompareVersions = function(aV1, aV2) {
var v1 = aV1.split(".");
var v2 = aV2.split(".");
var numSubversions = (v1.length > v2.length) ? v1.length : v2.length;
for (var i = 0; i < numSubversions; i++) {
if (typeof v2[i] == 'undefined') {
return 1;
}
if (typeof v1[i] == 'undefined') {
return -1;
}
if (parseInt(v2[i]) > parseInt(v1[i])) {
return -1;
} else if (parseInt(v2[i]) < parseInt(v1[i])) {
return 1;
}
}
return 0;
}
G_ExtensionUpdater.prototype.GetCurrent = function() {
var container   = Cc["@mozilla.org/rdf/container;1"]
.getService(Ci.nsIRDFContainer);
var extensionDS = Cc["@mozilla.org/extensions/manager;1"]
.getService(Ci.nsIExtensionManager).datasource;
var root = this.RDFSvc_.GetResource("urn:mozilla:item:root");
var moz = "http://www.mozilla.org/2004/em-rdf#";
var nameArc = this.RDFSvc_.GetResource(moz + "name");
var versionArc = this.RDFSvc_.GetResource(moz + "version");
var updateURLArc = this.RDFSvc_.GetResource(moz + "updateURL");
G_Debug(this, "Init container...");
container.Init(extensionDS, root);
var found = false;
var elements = container.GetElements();
while (elements.hasMoreElements()) {
var name = "";
var version = "";
var updateURL = "";
var element = elements.getNext().QueryInterface(Ci.nsIRDFResource);
var nameRes = extensionDS.GetTarget(element, nameArc ,true);
if (nameRes) {
name = nameRes.QueryInterface(Ci.nsIRDFLiteral).Value;
G_Debug(this, "Installed: %s".subs(name));
}
var versionRes = extensionDS.GetTarget(element, versionArc ,true);
if (versionRes) {
version = versionRes.QueryInterface(Ci.nsIRDFLiteral).Value;
G_Debug(this, "Version: %s".subs(version));
}
var updateURLRes = extensionDS.GetTarget(element, updateURLArc ,true);
if (updateURLRes) {
updateURL = updateURLRes.QueryInterface(Ci.nsIRDFLiteral).Value;
G_Debug(this, "Update URL: %s".subs(updateURL));
}
if (updateURL && (name.toLowerCase() == this.name.toLowerCase())) {
G_Debug(this, "Found %s\t%s".subs(name, updateURL));
this.currentUpdateURL = updateURL;
this.currentVersion = version;
return true;
}
}
return false;
}
G_ExtensionUpdater.prototype.AttemptUpdate = function() {
if (!this.currentUpdateURL) {
return false;
}
var ds = this.RDFSvc_.GetDataSource(this.currentUpdateURL);
var remote = ds.QueryInterface(Ci.nsIRDFRemoteDataSource);
if (remote.loaded) {
G_Debug(this, "the datasource was already loaded! (somehow)");
this.onEndLoad();
} else {
G_Debug(this, "the datasource wasn't loaded, but it's loading now!");
remote.QueryInterface(Ci.nsIRDFXMLSink).addXMLSinkObserver(this);
}
}
G_ExtensionUpdater.prototype.onBeginLoad = function(aSink) {;}
G_ExtensionUpdater.prototype.onInterrupt = function(aSink) {;}
G_ExtensionUpdater.prototype.onResume = function(aSink) {;}
G_ExtensionUpdater.prototype.onError = function(aSink, aStatus, aErrorMsg) {
this.Failure("RDF retrieval Error: \"%s\"".subs(aErrorMsg));
}
G_ExtensionUpdater.prototype.onEndLoad = function(aSink) {
G_Debug(this, "RDF loaded");
var moz = "http://www.mozilla.org/2004/em-rdf#";
var versionArc = this.RDFSvc_.GetResource(moz + "version");
var updateLinkArc = this.RDFSvc_.GetResource(moz + "updateLink");
var thisResource = null;
var dsResources = aSink.GetAllResources();
while (dsResources.hasMoreElements()) {
thisResource = dsResources.getNext().QueryInterface(Ci.nsIRDFResource);
var versionRes = aSink.GetTarget(thisResource, versionArc, true);
if (versionRes) {
this.updateVersion = versionRes.QueryInterface(Ci.nsIRDFLiteral).Value;
}
var updateLinkRes = aSink.GetTarget(thisResource, updateLinkArc, true);
if (updateLinkRes) {
this.updateLink = updateLinkRes.QueryInterface(Ci.nsIRDFLiteral).Value;
}
}
if (this.updateVersion && this.updateLink) {
G_Debug(this, "currentVersion:%s\nupdateVersion: %s\nupdateLink: %s"
.subs(this.currentVersion,
this.updateVersion,
this.updateLink));
if (this.CompareVersions(this.updateVersion, this.currentVersion) == 1) {
G_Debug(this, "Local version is old, now installing update...");
this.InstallUpdate();
} else {
G_Debug(this, "No need to update");
}
}
}
G_ExtensionUpdater.prototype.InstallUpdate = function() {
if (!this.updateLink) {
this.Failure("Failure");
return false;
}
var manager = Cc["@mozilla.org/xpinstall/install-manager;1"]
.createInstance(Ci.nsIXPInstallManager);
if (manager != null) {
var items = [this.updateLink];
var installProgress = {
onStateChange: function(index, state, value) {;},
onProgress: function(index, value, maxValue) {;}
};
var p = new G_Preferences();
var autoupdate = p.getPref("extensions.google.%s.autoupdate"
.subs(this.name), false);
if (autoupdate == false) {
G_DebugL(this, "Extension '%s' would've been updated".subs(this.name));
} else {
G_DebugL(this, "Extension '%s' updating...".subs(this.name));
manager.initManagerFromChrome(items, items.length, installProgress);
}
} else {
G_Debug(this, "Error creating manager");
}
this.Success();
}
G_ExtensionUpdater.prototype.Failure = function(aMessage) {
G_Debug(this, "Failure: %s".subs(aMessage));
this.updating = false;
this.OnFail(aMessage);
}
G_ExtensionUpdater.prototype.Success = function() {
this.updating = false;
this.OnSuccess();
}
G_ExtensionUpdater.prototype.OnFail = function(aMessage) {}
G_ExtensionUpdater.prototype.OnSuccess = function() {}
G_ExtensionUpdater.prototype.debugZone = "G_ExtensionUpdater";
function TEST_G_ExtensionUpdater() {
if (G_GDEBUG) {
var z = "G_ExtensionUpdater UNITTEST";
G_debugService.enableZone(z);
G_Debug(z, "Starting");
var name = "111. This name shouldn't exist";
var upd = new G_ExtensionUpdater(name);
G_Assert(z, name == upd.name, "Name not inserted correctly.");
G_Assert(z, !upd.GetCurrent(),
"Extension found when shouldn't have.");
function testCompare(a, b, expected) {
}
var a, b;
a = "1.222.00";
b = "1.222.01";
G_Assert(z, upd.CompareVersions(a, b) == -1,
"CompareVersions says %s is not higher than %s".subs(b, a));
a = "1.22.00";
b = "1.222.01";
G_Assert(z, upd.CompareVersions(a, b) == -1,
"CompareVersions says %s is not higher than %s".subs(b, a));
a = "1.9";
b = "1.1.1101";
G_Assert(z, upd.CompareVersions(a, b) == 1,
"CompareVersions says %s is not higher than %s".subs(a, b));
G_Assert(z, upd.CompareVersions(b, a) == -1,
"CompareVersions says %s is not higher than %s".subs(b, a));
a = "1.1.1101";
G_Assert(z, upd.CompareVersions(a, a) == 0,
"CompareVersions says %s is not equal to %s".subs(a, a));
a = "4.121";
G_Assert(z, upd.CompareVersions(a, a) == 0,
"CompareVersions says %s is not equal to %s".subs(a, a));
a = "1.0000.001.121";
b = "1.000.01.121";
G_Assert(z, upd.CompareVersions(a, b) == 0,
"CompareVersions says %s is not equal to %s".subs(a, b));
G_Debug(z, "PASSED");
}
}
