function PROT_ListManager(threadQueue, opt_testing) {
this.debugZone = "listmanager";
if (opt_testing)
this.debugZone += "testing";
G_debugService.enableZone(this.debugZone);
this.threadQueue_ = threadQueue;
this.testing_ = !!opt_testing;
this.appDir_ = null;                 // appDir is set via method
this.rpcPending_ = false;
this.readingData_ = false;
this.dataImportedAtLeastOnce_ = false;
var self = this;
new G_Alarm(function() { self.dataImportedAtLeastOnce_ = true; }, 60 * 1000);
this.updateserverURL_ = PROT_globalStore.getUpdateserverURL();
this.tablesKnown = {
"goog-white-domain": new PROT_VersionParser("goog-white-domain", 1, -1),
"goog-black-url" : new PROT_VersionParser("goog-black-url", 1, -1),
"goog-black-enchash": new PROT_VersionParser("goog-black-enchash", 1, -1),
"test1-foo-domain" : new PROT_VersionParser("test1-foo-domain", 0, -1),
"test2-foo-domain" : new PROT_VersionParser("test2-foo-domain", 0, -1),
};
this.tablesData = {};
this.prefs_ = new G_Preferences();
}
PROT_ListManager.prototype.maybeStartManagingUpdates = function() {
if (this.testing_)
return;
var checkRemotePrefName = PROT_globalStore.getServerCheckEnabledPrefName();
this.checkRemote_ = this.prefs_.getPref(checkRemotePrefName, null);
if (!this.registeredPrefObservers_) {
var checkRemotePrefObserver = BindToObject(this.onCheckRemotePrefChanged,
this);
this.prefs_.addObserver(checkRemotePrefName, checkRemotePrefObserver);
this.registeredPrefObservers_ = true;
}
if (this.checkRemote_ === false) {
new G_Alarm(BindToObject(this.readDataFiles, this), 0);
new G_Alarm(BindToObject(this.checkForUpdates, this), 3000);
this.startUpdateChecker();
}
}
PROT_ListManager.prototype.onCheckRemotePrefChanged = function(prefName) {
var oldValue = this.checkRemote_;
var value = this.prefs_.getBoolPrefOrDefault(prefName, null);
if (value !== null)
this.checkRemote_ = value;
G_Debug(this,
"CheckRemote Pref Changed: " + prefName +
" = " + this.checkRemote_);
if (this.testing_)
return;
if (this.checkRemote_ === false) {
new G_Alarm(BindToObject(this.readDataFiles, this), 0);
new G_Alarm(BindToObject(this.checkForUpdates, this), 3000);
this.startUpdateChecker();
} else if (oldValue === false && this.checkRemote_) {
this.stopUpdateChecker();                    // Cancel pending updates
}
}
PROT_ListManager.prototype.startUpdateChecker = function() {
this.stopUpdateChecker();
var thirtyMinutes = 30 * 60 * 1000;
this.updateChecker_ = new G_Alarm(BindToObject(this.checkForUpdates, this),
thirtyMinutes, true /* repeat */);
}
PROT_ListManager.prototype.stopUpdateChecker = function() {
if (this.updateChecker_)
this.updateChecker_.cancel();
}
PROT_ListManager.prototype.setAppDir = function(appDir) {
this.appDir_ = appDir;
}
PROT_ListManager.prototype.clearList = function(table) {
if (!this.tablesKnown[table])
return false;
this.tablesData[table] = new PROT_TRTable(this.tablesKnown[table].type);
return true;
}
PROT_ListManager.prototype.safeLookup = function(table, key) {
var result = false;
try {
var map = this.tablesData[table];
result = map.find(key);
} catch(e) {
result = false;
G_Debug(this, "Safelookup masked failure: " + e);
}
return result;
}
PROT_ListManager.prototype.safeInsert = function(table, key, value) {
if (!this.tablesKnown[table]) {
G_Debug(this, "Unknown table: " + table);
return false;
}
if (!this.tablesData[table])
this.tablesData[table] = new PROT_TRTable(table);
try {
this.tablesData[table].insert(key, value);
} catch (e) {
G_Debug(this, "Cannot insert key " + key + " value " + value);
G_Debug(this, e);
return false;
}
return true;
}
PROT_ListManager.prototype.safeErase = function(table, key) {
if (!this.tablesKnown[table]) {
G_Debug(this, "Unknown table: " + table);
return false;
}
if (!this.tablesData[table])
return false;
return this.tablesData[table].erase(key);
}
PROT_ListManager.prototype.readDataFiles = function() {
if (this.rpcPending_) {
G_Debug(this, "Cannot read data files while an update RPC is pending");
return false;
}
this.readingData_ = true;
var files = [];
for (var type in this.tablesKnown) {
var filename = type + ".sst";
var file = this.appDir_.clone();
file.append(filename);
if (file.exists() && file.isFile() && file.isReadable()) {
G_Debug(this, "Found saved data for: " + type);
files.push(file);
} else {
G_Debug(this, "Failed to find saved data for: " + type);
}
}
var data = "";
for (var i = 0; i < files.length; ++i) {
G_Debug(this, "Trying to read: " + files[i].path);
var gFile = new G_FileReader(files[i]);
data += gFile.read() + "\n";
gFile.close();
}
var wfr = new PROT_WireFormatReader(this.threadQueue_, this.tablesData);
wfr.deserialize(data, BindToObject(this.dataFromDisk, this));
return true;
}
PROT_ListManager.prototype.dataFromDisk = function(tablesKnown, tablesData) {
G_Debug(this, "Called dataFromDisk");
this.importData_(tablesKnown, tablesData);
this.readingData_ = false;
return true;
}
PROT_ListManager.prototype.getRequestURL_ = function(url) {
url += "version=";
var firstElement = true;
for(var type in this.tablesKnown) {
if (this.tablesKnown[type].major == 0)
continue;
if (!firstElement) {
url += ","
} else {
firstElement = false;
}
url += type + ":" + this.tablesKnown[type].toUrl();
}
G_Debug(this, "Returning: " + url);
return url;
}
PROT_ListManager.prototype.checkForUpdates = function() {
if (this.rpcPending_) {
G_Debug(this, 'checkForUpdates: old callback is still pending...');
return false;
}
if (this.readingData_) {
G_Debug(this, 'checkForUpdate: still reading data from disk...');
new G_Alarm(BindToObject(this.checkForUpdates, this), 500);
return false;
}
G_Debug(this, 'checkForUpdates: scheduling request..');
this.rpcPending_ = true;
this.xmlFetcher_ = new PROT_XMLFetcher();
this.xmlFetcher_.get(this.getRequestURL_(this.updateserverURL_),
BindToObject(this.rpcDone, this));
return true;
}
PROT_ListManager.prototype.rpcDone = function(data) {
G_Debug(this, "Called rpcDone");
var wfr = new PROT_WireFormatReader(this.threadQueue_, this.tablesData);
wfr.deserialize(data, BindToObject(this.dataReady, this));
this.xmlFetcher_ = null;
}
PROT_ListManager.prototype.hasData = function() {
return !!this.dataImportedAtLeastOnce_;
}
PROT_ListManager.prototype.importData_ = function(tablesKnown, tablesData) {
this.dataImportedAtLeastOnce_ = true;
var changes = [];
if (tablesKnown && tablesData) {
for (var name in tablesKnown) {
if (this.tablesKnown[name] != tablesKnown[name]) {
changes.push(name);
this.tablesKnown[name] = tablesKnown[name];
this.tablesData[name] = tablesData[name];
}
}
}
return changes;
}
PROT_ListManager.prototype.dataReady = function(tablesKnown, tablesData) {
G_Debug(this, "Called dataReady");
var changes = this.importData_(tablesKnown, tablesData);
if (changes.length) {
G_Debug(this, "Commiting " + changes.length + " changed tables to disk.");
for (var i = 0; i < changes.length; i++)
this.storeTable(changes[i],
this.tablesData[changes[i]],
this.tablesKnown[changes[i]]);
}
this.rpcPending_ = false;            // todo maybe can do away cuz asynch
G_Debug(this, "Done writing data to disk.");
}
PROT_ListManager.prototype.storeTable = function(tableName,
opt_table,
opt_parser) {
var table = opt_table ? opt_table : this.tablesData[tableName];
var parser = opt_parser ? opt_parser : this.tablesKnown[tableName];
if (!table || ! parser)
G_Error(this, "Tried to serialize a non-existent table: " + tableName);
var wfw = new PROT_WireFormatWriter(this.threadQueue_);
wfw.serialize(table, parser, BindToObject(this.writeDataFile,
this,
tableName));
}
PROT_ListManager.prototype.writeDataFile = function(tableName, tableData) {
var filename = tableName + ".sst";
G_Debug(this, "Serializing to " + filename);
try {
var tmpFile = G_File.createUniqueTempFile(filename);
var tmpFileWriter = new G_FileWriter(tmpFile);
tmpFileWriter.write(tableData);
tmpFileWriter.close();
} catch(e) {
G_Debug(this, e);
G_Error(this, "Couldn't write to temp file: " + filename);
}
try {
tmpFile.moveTo(this.appDir_, filename);
} catch(e) {
G_Debug(this, e);
G_Error(this, "Couldn't overwrite existing table: " +
this.appDir_.path + filename);
tmpFile.remove(false /* not recursive */);
}
G_Debug(this, "Serializing to " + filename + " finished.");
}
function TEST_PROT_ListManager() {
if (G_GDEBUG) {
var z = "listmanager UNITTEST";
G_debugService.enableZone(z);
G_Debug(z, "Starting");
var tempDir = G_File.createUniqueTempDir();
var listManager = new PROT_ListManager(new TH_ThreadQueue(),
true /*testing*/);
listManager.setAppDir(tempDir);
var data = "";
var set1Name = "test1-foo-domain";
data += "[" + set1Name + " 1.2]\n";
var set1 = {};
for (var i = 0; i < 10; i++) {
set1["http://" + i + ".com"] = 1;
data += "+" + i + ".com\t1\n";
}
var set2Name = "test2-foo-domain";
data += "\n[" + set2Name + " 1.7]\n";
var set2 = {};
for (var i = 0; i < 5; i++) {
set2["http://" + i + ".com"] = 1;
data += "+" + i + ".com\t1\n";
}
function deserialized(tablesKnown, tablesData) {
listManager.dataReady(tablesKnown, tablesData);
var file = tempDir.clone();
file.append(set1Name + ".sst");
G_Assert(z, file.exists() && file.isFile() && file.isReadable(),
"Failed to write out: " + file.path);
file = tempDir.clone();
file.append(set2Name + ".sst");
G_Assert(z, file.exists() && file.isFile() && file.isReadable(),
"Failed to write out: " + file.path);
listManager = new PROT_ListManager(new TH_ThreadQueue(),
true /*testing*/);
listManager.setAppDir(tempDir);
listManager.readDataFiles();
for (var prop in set1)
G_Assert(z,
listManager.tablesData[set1Name].find(prop) == 1,
"Couldn't find member " + prop + "of set1 from disk.");
for (var prop in set2)
G_Assert(z,
listManager.tablesData[set2Name].find(prop) == 1,
"Couldn't find member " + prop + "of set2 from disk.");
tempDir.remove(true);
G_Debug(z, "PASSED");
};
}
var wfr = new PROT_WireFormatReader(new TH_ThreadQueue,
listManager.tablesData);
wfr.deserialize(data, deserialized);
}
