function PROT_TRTable(name) {
G_Map.call(this, name);
this.debugZone = "trtable";
this.enchashDecrypter_ = new PROT_EnchashDecrypter();
var components = name.split("-");
if (components.length != 3)
throw new Error("TRTables have names like name-function-type");
if (components[2] == "url")
this.find = this.lookupURL_;
else if (components[2] == "domain")
this.find = this.lookupDomain_;
else if (components[2] == "enchash")
this.find = this.lookupEnchash_;
else
throw new Error("Unknown table type: " + components[2]);
}
PROT_TRTable.inherits(G_Map);
PROT_TRTable.prototype.lookupURL_ = function(url) {
var canonicalized = PROT_URLCanonicalizer.canonicalizeURL_(url);
return this.find_(url);
}
PROT_TRTable.prototype.lookupDomain_ = function(url) {
var urlObj = Cc["@mozilla.org/network/standard-url;1"]
.createInstance(Ci.nsIURL);
urlObj.spec = url;
var host = urlObj.host;
var components = host.split(".");
for (var i = 0; i < components.length - 1; i++) {
host = components.slice(i).join(".");
var val = this.find_(host);
if (val)
return val;
}
return undefined;
}
PROT_TRTable.prototype.lookupEnchash_ = function(url) {
var host = this.enchashDecrypter_.getCanonicalHost(url);
for (var i = 0; i < PROT_EnchashDecrypter.MAX_DOTS + 1; i++) {
var key = this.enchashDecrypter_.getLookupKey(host);
var encrypted = this.find_(key);
if (encrypted) {
G_Debug(this, "Enchash DB has host " + host);
var decrypted = this.enchashDecrypter_.decryptData(encrypted, host);
var res = this.enchashDecrypter_.parseRegExps(decrypted);
for (var j = 0; j < res.length; j++)
if (res[j].test(url))
return true;
}
var index = host.indexOf(".");
if (index == -1)
break;
host = host.substring(index + 1);
}
return undefined;
}
function TEST_PROT_TRTable() {
if (G_GDEBUG) {
var z = "trtable UNITTEST";
G_debugService.enableZone(z);
G_Debug(z, "Starting");
var url = "http://www.yahoo.com?foo=bar";
var urlTable = new PROT_TRTable("test-foo-url");
urlTable.insert(url, "1");
G_Assert(z, urlTable.find(url) == "1", "URL lookups broken");
G_Assert(z, !urlTable.find("about:config"), "about:config breaks domlook");
var dom1 = "bar.com";
var dom2 = "amazon.co.uk";
var dom3 = "127.0.0.1";
var domainTable = new PROT_TRTable("test-foo-domain");
domainTable.insert(dom1, "1");
domainTable.insert(dom2, "1");
domainTable.insert(dom3, "1");
G_Assert(z, domainTable.find("http://www.bar.com/?zaz=asdf#url") == "1",
"Domain lookups broken (single dot)");
G_Assert(z, domainTable.find("http://www.amazon.co.uk/?z=af#url") == "1",
"Domain lookups broken (two dots)");
G_Assert(z, domainTable.find("http://127.0.0.1/?z=af#url") == "1",
"Domain lookups broken (IP)");
G_Debug(z, "PASSED");
}
}
