function PROT_PhishingWarden(listManager, opt_testing) {
this.debugZone = "phishwarden";
this.listManager_ = listManager;
this.testing_ = !!opt_testing;
this.browserViews_ = [];
this.displayers_ = {
"afterload": PROT_PhishMsgDisplayer,
};
this.fetcher_ = new PROT_TRFetcher();
if (!this.testing_) {
this.navWatcher_ = new G_NavWatcher(true /* filter spurious navs */);
this.navWatcher_.registerListener("docnavstart",
BindToObject(this.onDocNavStart,
this));
}
this.prefs_ = new G_Preferences();
var checkRemotePrefName = PROT_globalStore.getServerCheckEnabledPrefName();
this.checkRemote_ = this.prefs_.getPref(checkRemotePrefName, null);
var checkRemotePrefObserver = BindToObject(this.onCheckRemotePrefChanged,
this);
this.prefs_.addObserver(checkRemotePrefName, checkRemotePrefObserver);
var phishWardenPrefName = PROT_globalStore.getPhishWardenEnabledPrefName();
this.phishWardenEnabled_ = this.prefs_.getPref(phishWardenPrefName, null);
var phishWardenPrefObserver =
BindToObject(this.onPhishWardenEnabledPrefChanged, this);
this.prefs_.addObserver(phishWardenPrefName, phishWardenPrefObserver);
this.testURL_ = PROT_globalStore.getTestURL();
}
PROT_PhishingWarden.prototype.addBrowserView = function(view) {
G_Debug(this, "New browser view registered.");
this.browserViews_.push(view);
}
PROT_PhishingWarden.prototype.removeBrowserView = function(view) {
for (var i = 0; i < this.browserViews_.length; i++)
if (this.browserViews_[i] === view) {
G_Debug(this, "Browser view unregistered.");
this.browserViews_.splice(i, 1);
return;
}
G_Assert(this, false, "Tried to unregister non-existent browser view!");
}
PROT_PhishingWarden.prototype.onCheckRemotePrefChanged = function(prefName) {
this.checkRemote_ = this.prefs_.getBoolPrefOrDefault(prefName,
this.checkRemote_);
}
PROT_PhishingWarden.prototype.onPhishWardenEnabledPrefChanged = function(
prefName) {
this.phishWardenEnabled_ =
this.prefs_.getBoolPrefOrDefault(prefName, this.phishWardenEnabled_);
}
PROT_PhishingWarden.prototype.onDocNavStart = function(e) {
var url = e.url;
var request = e.request;
G_Debug(this, "phishWarden: " +
(this.phishWardenEnabled_ ? "enabled" : "disabled"));
G_Debug(this, "checkRemote: " +
(this.checkRemote_ ? "yes" : "no"));
G_Debug(this, "isTestURL: " +
(this.isBlacklistTestURL(url) ? "yes" : "no"));
if (this.isBlacklistTestURL(url) &&
(this.phishWardenEnabled_ === true ||
this.phishWardenEnabled_ === null)) {
this.houstonWeHaveAProblem_(request);
} else if (this.phishWardenEnabled_ === true) {
if (this.checkRemote_) {
this.fetcher_.get(url,
BindToObject(this.onTRFetchComplete,
this,
request));
} else {
if (!this.listManager_.hasData()) {
G_Debug(this, "Listmanager hasn't loaded local list. Delaying check.");
new G_Alarm(BindToObject(this.onDocNavStart, this, e), 1000);
} else {
var callback = BindToObject(this.houstonWeHaveAProblem_,
this,
request);
this.checkURL(callback, url);
}
}
}
}
PROT_PhishingWarden.prototype.onTRFetchComplete = function(request,
trValues) {
var callback = BindToObject(this.houstonWeHaveAProblem_, this, request);
this.checkRemoteData(callback, trValues);
}
PROT_PhishingWarden.prototype.houstonWeHaveAProblem_ = function(request) {
if (this.maybeLocateProblem_(request))       // Cases 1 and 2
return;
if (request.isPending()) {        // Case 3
G_Debug(this, "Can't find problem Doc; Req pending. Retrying.");
new G_Alarm(BindToObject(this.houstonWeHaveAProblem_,
this,
request),
200 /*ms*/);
} else {                          // Case 4
G_Debug(this,
"Can't find problem Doc; Req completed. Retrying at most twice.");
new G_ConditionalAlarm(BindToObject(this.maybeLocateProblem_,
this,
request),
0 /* next event loop */,
true /* repeat */,
2 /* at most twice */);
}
}
PROT_PhishingWarden.prototype.maybeLocateProblem_ = function(request) {
for (var i = 0; i < this.browserViews_.length; i++)
if (this.browserViews_[i].tryToHandleProblemRequest(this, request)) {
G_Debug(this, "Found browser view willing to handle problem!");
return true;
}
return false;
}
PROT_PhishingWarden.prototype.isBlacklistTestURL = function(url) {
return url === this.testURL_;
}
PROT_PhishingWarden.prototype.checkURL = function(callback, url) {
G_Debug(this, "Checking URL for " + url);
if (this.isEvilURL_(url) || this.isBlacklistTestURL(url)) {
callback(this);
G_Debug(this, "Local blacklist hit");
(new PROT_Reporter).report("phishblhit", url);
return;
}
G_Debug(this, "Local blacklist miss");
}
PROT_PhishingWarden.prototype.checkRemoteData = function(callback,
trValues) {
if (!trValues) {
G_Debug(this, "Didn't get TR values from the server.");
return;
}
G_Debug(this, "Page has phishiness " + trValues["phishy"]);
if (trValues["phishy"] == 1) {     // It's on our blacklist
G_Debug(this, "Remote blacklist hit");
callback(this);
} else {
G_Debug(this, "Remote blacklist miss");
}
}
PROT_PhishingWarden.prototype.checkDocument = function(callback,
doc) {
}
PROT_PhishingWarden.prototype.isWhiteURL_ = function(url) {
if (!this.listManager_)
return false;
return this.listManager_.safeLookup("goog-white-domain", url);
}
PROT_PhishingWarden.prototype.isBlackURL_ = function(url) {
if (!this.listManager_)
return false;
var isBlack = this.listManager_.safeLookup("goog-black-url", url) ||
this.listManager_.safeLookup("goog-black-enchash", url);
return isBlack;
}
PROT_PhishingWarden.prototype.isEvilURL_ = function(url) {
return !this.isWhiteURL_(url) && this.isBlackURL_(url);
}
function TEST_PROT_PhishingWarden() {
if (G_GDEBUG) {
var z = "phishwarden UNITTEST";
G_debugService.enableZone(z);
G_Debug(z, "Starting");
var threadQueue = new TH_ThreadQueue();
var listManager = new PROT_ListManager(threadQueue, true /* testing */);
var warden = new PROT_PhishingWarden(listManager, true /* testing */);
var blacklistedCount = 0;
function shouldNotCall() {
G_Assert(z, false, "Blacklist lookup error");
};
function shouldCall() {
blacklistedCount++;
G_Assert(z, blacklistedCount <= 4, "More than four blacklist hits");
};
var blackURLs = [
"http://foo.com/1",
"http://foo.com/2",
"http://foo.com/3",
"http://foo.com/4",
];
for (var i = 0; i < blackURLs.length; i++)
listManager.safeInsert("goog-black-url", blackURLs[i], "1");
warden.checkURL(shouldNotCall, "http://bar.com/");
warden.checkURL(shouldCall, "http://foo.com/1");
warden.checkURL(shouldCall, "http://foo.com/2");
warden.checkURL(shouldCall, "http://foo.com/3");
warden.checkURL(shouldCall, "http://foo.com/4");
G_Assert(z, blacklistedCount == 4, "Blacklist lookup failure");
G_Debug(z, "PASSED");
}
}
